package com.softmoore.math;


object MatrixUtil
  {
    def multiply(a : Array[Array[Double]], b : Array[Array[Double]]) : Array[Array[Double]] =
      {
        if (!checkArgs(a, b))
            throw new IllegalArgumentException("Matrices not compatible for multiplication");
        
        val nRows : Int = a.length;
        val nCols : Int = b(0).length;
        
        var result = Array.ofDim[Double](nRows, nCols);

        for (rowNum <- 0 until nRows)
          {
            for (colNum <- 0 until nCols)
              {
                var sum : Double = 0.0;
                
                for (i <- 0  until a(0).length)
                    sum += a(rowNum)(i)*b(i)(colNum);

                result(rowNum)(colNum) = sum;
              }
          }
        
        return result;
      }


    /**
     * Checks that the two specified matrices are compatible for multiplication.
     * @return true if the matrix product a*b is possible;
     */
    def checkArgs(a : Array[Array[Double]], b : Array[Array[Double]]) : Boolean =
      {
        // check that matrices are not null
        if (a == null || b ==  null)
            return false;
        
        // check that we actually have columns
        if (a(0) == null || b(0) == null)
            return false;

        // check that all columns of matrix a have the same size
        val aColSize : Int = a(0).length;
        for (i <- 0 until a.size)
          {
            if (a(i) == null || a(i).length != aColSize)
                return false;
          }

        // check that all columns of matrix b have the same size
        val bColSize : Int = b(0).length;
        for (i <- 0 until b.size)
          {
            if (b(i) == null || b(i).length != bColSize)
                return false;
          }

        // check that number of columns of matrix a equals the number of rows of matrix b
        if (aColSize != b.length)
            return false;

        return true;
      }
  }
